package com.aliyun.odps.spark.examples.makedata_ad.v20240718

import com.aliyun.odps.data.Record
import com.aliyun.odps.spark.examples.myUtils.{MyDateUtils, ParamUtils, env}
import com.aliyun.odps.{Column, TableSchema}
import org.apache.spark.sql.SparkSession

object makedata_ad_33_diffFeature_20250708 {
  def main(args: Array[String]): Unit = {
    val spark = SparkSession
      .builder()
      .appName(this.getClass.getName)
      .getOrCreate()
    val sc = spark.sparkContext

    val param = ParamUtils.parseArgs(args)
    val project = param.getOrElse("project", "loghubods")
    val tablePart = param.getOrElse("tablePart", "64").toInt
    val beginStr = param.getOrElse("beginStr", "20250708")
    val endStr = param.getOrElse("endStr", "20250708")

    val odpsOps = env.getODPS(sc)
    val dateRange = MyDateUtils.getDateRange(beginStr, endStr)
    for (dt <- dateRange) {
      val partition = s"dt=$dt"
      // 读取样本表
      val sampleRdd = odpsOps.readTable(
        project = project,
        table = "ad_easyrec_train_realtime_data_v3_sampled",
        partition = partition,
        transfer = func,
        numPartition = tablePart
      )

      // 读取特征表，并按 mid 去重（保留最后一条）
      val sampleRdd1 = odpsOps.readTable(
        project = project,
        table = "ad_easyrec_train_realtime_data_v3_sampled_temp",
        partition = partition,
        transfer = func,
        numPartition = tablePart
      )


      val recordRdd = sampleRdd.join(sampleRdd1).map {
        case (pqtId, (sampleMap, sample1Map)) =>
          // 合并两个 Map 的所有键
          val allKeys = sampleMap.keySet ++ sample1Map.keySet

          // 统计每个键在两个 Map 中的值差异
          val diffCounts = allKeys.map { key =>
            val valueInSample = sampleMap.get(key)
            val valueInSample1 = sample1Map.get(key)

            // 判断值是否不同（包括存在性和实际值）
            val isDifferent = (valueInSample, valueInSample1) match {
              case (None, Some(_)) => true  // 键只存在于 sample1Map
              case (Some(_), None) => true  // 键只存在于 sampleMap
              case (Some(v1), Some(v2)) => v1 != v2  // 键存在但值不同
              case _ => false  // 都不存在或都相同
            }

            (key, if (isDifferent) 1 else 0)  // 不同则计为1，相同则计为0
          }.toMap

          // 返回 patId 和差异统计结果
          (pqtId, diffCounts)
      }

      // 按 key 聚合所有差异数量
      val keyDiffCounts = recordRdd
        .flatMap { case (_, diffCounts) => diffCounts }  // 展开所有差异统计
        .reduceByKey(_ + _)  // 按 key 累加差异数量

      // 输出结果
      keyDiffCounts.collect().foreach { case (key, count) =>
        println(s"Key: $key, 差异数量: $count")
      }


    }


  }

  def func(record: Record, schema: TableSchema): (String, Map[String, String]) = {
    // 1. 获取所有列信息
    val columns: Array[Column] = schema.getColumns.toArray(Array.empty[Column])

    // 2. 遍历列，找到 "mid" 字段的索引
    var pdtIdIndex = -1
    for (i <- columns.indices) {
      if (columns(i).getName == "pqtid") {
        pdtIdIndex = i
      }
    }

    // 4. 处理 mid 字段（支持非字符串类型转换为字符串）
    val midColumn = columns(pdtIdIndex)
    val midType = midColumn.getTypeInfo.getTypeName // 获取 mid 字段类型
    val midValue: Any = midType match {
      case "STRING" => record.getString(pdtIdIndex)
      case "BIGINT" => record.getBigint(pdtIdIndex)  // 长整型
      case "DOUBLE" => record.getDouble(pdtIdIndex)  // 浮点型
      case "BOOLEAN" => record.getBoolean(pdtIdIndex) // 布尔型
    }
    val mid = Option(midValue).map(_.toString).getOrElse("") // 转换为字符串，null 转为空字符串

    // 5. 处理所有字段（非字符串类型转为字符串）
    val recordMap = columns.zipWithIndex
      .map { case (column, index) =>
        val columnName = column.getName
        val columnType = column.getTypeInfo.getTypeName // 获取字段类型

        // 根据字段类型获取值并转换为字符串
        val value: String = columnType match {
          case "STRING" =>
            val str = record.getString(index)
            if (str == null) null else str // 字符串类型直接保留（null 保持 null）

          case "BIGINT" =>
            val num = record.getBigint(index)
            if (num == null) null else num.toString // 长整型转字符串

          case "DOUBLE" =>
            val num = record.getDouble(index)
            if (num == null) null else num.toString // 浮点型转字符串

          case "BOOLEAN" =>
            val bool = record.getBoolean(index)
            if (bool == null) null else bool.toString // 布尔型转字符串（"true"/"false"）
        }

        columnName -> value
      }
      .toMap

    // 6. 返回 (mid, Map[String, String])
    (mid, recordMap)
  }

}
